import { db } from "@/lib/db"
import { NextResponse } from "next/server";
import fetchUserWithImpersonation from "@/lib/fetch-user-impersonation";

export async function GET(req: Request) {
    try {
        const user = await fetchUserWithImpersonation(req)
        if (!user) {
            return NextResponse.json({ message: 'Unauthorized.', error: null, value: null }, { status: 401 });
        }

        const filters = await db.ttsWordFilter.findMany({
            where: {
                userId: user.id
            }
        });

        return NextResponse.json(filters);
    } catch (error) {
        console.log("[TTS/FILTER/WORDS]", error);
        return NextResponse.json({ message: 'Something went wrong', error: error, value: null }, { status: 500 })
    }
}

export async function POST(req: Request) {
    try {
        const user = await fetchUserWithImpersonation(req)
        if (!user) {
            return NextResponse.json({ message: 'Unauthorized.', error: null, value: null }, { status: 401 });
        }

        const { search, replace } = await req.json();
        if (!search || search.length < 3 || search.length > 127)
            return NextResponse.json({ message: 'search must be between 3 to 127 characters.', error: null, value: null }, { status: 400 })
        if (replace == null)
            return NextResponse.json({ message: 'replace must not be null', error: null, value: null }, { status: 400 })

        const filter = await db.ttsWordFilter.create({
            data: {
                search,
                replace,
                userId: user.id
            }
        });

        return NextResponse.json(filter);
    } catch (error) {
        console.log("[TTS/FILTER/WORDS]", error);
        return NextResponse.json({ message: 'Something went wrong', error: error, value: null }, { status: 500 })
    }
}

export async function PUT(req: Request) {
    try {
        const user = await fetchUserWithImpersonation(req)
        if (!user) {
            return NextResponse.json({ message: 'Unauthorized.', error: null, value: null }, { status: 401 });
        }

        const { id, search, replace } = await req.json();
        if (!id || id.length < 1)
            return NextResponse.json({ message: 'id does not exist.', error: null, value: null }, { status: 400 })
        if (!search || search.length < 3 || search.length > 127)
            return NextResponse.json({ message: 'search must be between 3 to 127 characters.', error: null, value: null }, { status: 400 })
        if (replace == null)
            return NextResponse.json({ message: 'replace must not be null.', error: null, value: null }, { status: 400 })

        const filter = await db.ttsWordFilter.update({
            where: {
                id
            },
            data: {
                search,
                replace,
                userId: user.id
            }
        });

        return NextResponse.json(filter);
    } catch (error) {
        return NextResponse.json({ message: 'Something went wrong', error: error, value: null }, { status: 500 })
    }
}

export async function DELETE(req: Request) {
    try {
        const user = await fetchUserWithImpersonation(req)
        if (!user) {
            return NextResponse.json({ message: 'Unauthorized.', error: null, value: null }, { status: 401 });
        }

        const { searchParams } = new URL(req.url)
        const id = searchParams.get('id') as string
        const search = searchParams.get('search') as string

        if (search) {
            if (!search || search.length < 3 || search.length > 127)
                return NextResponse.json({ message: 'search must be between 3 to 127 characters.', error: null, value: null }, { status: 400 })

            const filter = await db.ttsWordFilter.delete({
                where: {
                    userId_search: {
                        userId: user.id,
                        search
                    }
                }
            });

            return NextResponse.json(filter)
        } else if (id) {
            if (!id || id.length < 1)
                return NextResponse.json({ message: 'id does not exist.', error: null, value: null }, { status: 400 })

            const filter = await db.ttsWordFilter.delete({
                where: {
                    id: id
                }
            });

            return NextResponse.json(filter)
        }
        
        return NextResponse.json({ message: 'Either id or search must not be null.', error: null, value: null }, { status: 400 })
    } catch (error) {
        return NextResponse.json({ message: 'Something went wrong.', error: null, value: null }, { status: 500 })
    }
}